#include "stdafx.h"
#include "VolumeControl.h"
#include "utils.h"

int CVolumeControls::g_nVolumeStep, CVolumeControls::g_nBalanceStep;

CVolumeControl::CVolumeControl()
{
}

void CVolumeControl::Init(HWND hWnd, int MixerID)
{
	MMRESULT rc;				// Return code.
	MIXERLINE mxl;				// Holds the mixer line data.
	MIXERLINE mxl_con;

	m_hCallBackHwnd = hWnd;
	m_nCode = 0;
	m_hMixer = NULL;
	m_MixerID = MixerID;

	// open mixer
	rc = mixerOpen(&m_hMixer, m_MixerID, (DWORD_PTR)m_hCallBackHwnd, 0, CALLBACK_WINDOW | MIXER_OBJECTF_MIXER);
	if (rc != MMSYSERR_NOERROR)
	{
		utils::ErrorBox("Cannot open the mixer");
		m_nCode = 1;
		return;
	}

	// Initialize MIXERLINE structure.
	ZeroMemory(&mxl,sizeof(mxl));
	mxl.cbStruct = sizeof(MIXERLINE);
	mxl.dwDestination = 0;

	rc = mixerGetLineInfo((HMIXEROBJ)m_hMixer, &mxl, MIXER_GETLINEINFOF_DESTINATION);
	if (rc != MMSYSERR_NOERROR)
	{
		utils::ErrorBox("Cannot get the mixer line.");
		m_nCode = 2;
		return;
	}

	GetLineControls(mxl);

	for (unsigned int i = 0; i < mxl.cConnections; i++)
	{
		ZeroMemory(&mxl_con,sizeof(MIXERLINE));
		mxl_con.cbStruct		= sizeof(MIXERLINE);
		mxl_con.dwDestination	= 0;
		mxl_con.dwSource		= i;
		rc = mixerGetLineInfo((HMIXEROBJ)m_hMixer, &mxl_con, MIXER_GETLINEINFOF_SOURCE);
		if (rc != MMSYSERR_NOERROR)
			utils::ErrorBox("Cannot obtain line info");
		else
		{
			if (mxl_con.cControls > 0) GetLineControls(mxl_con);
		}
	}
}


///MEH NOT WORKING!?
void CVolumeControl::ReInit() {
	mixerClose(m_hMixer);

	MMRESULT rc;				// Return code.
	MIXERLINE mxl;				// Holds the mixer line data.
	MIXERLINE mxl_con;

	m_nCode = 0;
	m_hMixer = NULL;

	// open mixer
	rc = mixerOpen(&m_hMixer, m_MixerID, (DWORD_PTR)m_hCallBackHwnd, 0, CALLBACK_WINDOW | MIXER_OBJECTF_MIXER);
	if (rc != MMSYSERR_NOERROR)
	{
		utils::ErrorBox("Cannot open the mixer");
		m_nCode = 1;
		return;
	}

	// Initialize MIXERLINE structure.
	ZeroMemory(&mxl,sizeof(mxl));
	mxl.cbStruct = sizeof(MIXERLINE);
	mxl.dwDestination = 0;

	rc = mixerGetLineInfo((HMIXEROBJ)m_hMixer, &mxl, MIXER_GETLINEINFOF_DESTINATION);
	if (rc != MMSYSERR_NOERROR)
	{
		utils::ErrorBox("Cannot get the mixer line.");
		m_nCode = 2;
		return;
	}

	GetLineControls(mxl);

	for (unsigned int i = 0; i < mxl.cConnections; i++)
	{
		ZeroMemory(&mxl_con,sizeof(MIXERLINE));
		mxl_con.cbStruct		= sizeof(MIXERLINE);
		mxl_con.dwDestination	= 0;
		mxl_con.dwSource		= i;
		rc = mixerGetLineInfo((HMIXEROBJ)m_hMixer, &mxl_con, MIXER_GETLINEINFOF_SOURCE);
		if (rc != MMSYSERR_NOERROR)
			utils::ErrorBox("Cannot obtain line info");
		else
		{
			if (mxl_con.cControls > 0) GetLineControls(mxl_con);
		}
	}
}

CVolumeControl::~CVolumeControl()
{
	if (m_hMixer)
	{
		mixerClose(m_hMixer);
		m_vLines.clear();
		m_vMutes.clear();
	}
}


void CVolumeControl::GetLineControls(MIXERLINE &mxl)
{
	MMRESULT rc;
	MIXERLINECONTROLS mxlc;		// Obtains the mixer control.
	MIXERCONTROL *mxc = new MIXERCONTROL[mxl.cControls];

	ZeroMemory(&mxlc, sizeof(mxlc));
	mxlc.cbStruct	=	sizeof(mxlc);
	mxlc.dwLineID	=	mxl.dwLineID;
	mxlc.cControls	=	mxl.cControls;
	mxlc.cbmxctrl	=	sizeof(MIXERCONTROL);
	mxlc.pamxctrl	=	mxc;

	rc = mixerGetLineControls((HMIXEROBJ)m_hMixer, &mxlc, MIXER_GETLINECONTROLSF_ALL);
	if (rc != MMSYSERR_NOERROR)
	{
		utils::ErrorBox("Cannot get the control");
		return;
	}

	for (unsigned int i = 0; i < mxlc.cControls; i++)
	{
		switch (mxc[i].dwControlType)
		{
		case MIXERCONTROL_CONTROLTYPE_VOLUME:
			m_vLines.push_back(mxc[i]);
			break;
		case MIXERCONTROL_CONTROLTYPE_MUTE:
			m_vMutes.push_back(mxc[i]);
			break;
		}
	}
	delete mxc;
}



void CVolumeControl::GetVolume(int channel, int &left, int &right)
{
	
	MMRESULT rc;
	MIXERCONTROLDETAILS_UNSIGNED values[2];
	MIXERCONTROLDETAILS mxcd;
	ZeroMemory(&mxcd, sizeof(mxcd));
	mxcd.cbStruct		=	sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID	=	m_vLines[channel].dwControlID;
	mxcd.cChannels		=	2;
	mxcd.cMultipleItems	=	0;
	mxcd.cbDetails		=	sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	mxcd.paDetails		=	values;
	rc = mixerGetControlDetails((HMIXEROBJ)m_hMixer, &mxcd, MIXER_GETCONTROLDETAILSF_VALUE);
	if (rc != MMSYSERR_NOERROR)
	{		
		left = 0;
		right = 0;
		//ReInit();
		//utils::ErrorBox("Cannot get the current volume");
	}
	else
	{
		left = values[0].dwValue;
		right = values[1].dwValue;
	}
}


void CVolumeControl::SetVolume(int channel, int left, int right)
{
	MMRESULT rc;
	MIXERCONTROLDETAILS_UNSIGNED values[2];
	MIXERCONTROLDETAILS mxcd;

	values[0].dwValue = left;
	values[1].dwValue = right;

	ZeroMemory(&mxcd, sizeof(MIXERCONTROLDETAILS));
	mxcd.cbStruct		= sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID	= m_vLines[channel].dwControlID;
	mxcd.cChannels		= 2;
	mxcd.cMultipleItems	= 0;
	mxcd.cbDetails		= sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	mxcd.paDetails		= values;

	rc = mixerSetControlDetails((HMIXEROBJ)m_hMixer, &mxcd, MIXER_SETCONTROLDETAILSF_VALUE);
	if (rc != MMSYSERR_NOERROR)
	{
		utils::ErrorBox("Cannot set the volume");
	}
}


void CVolumeControl::SetMute(int channel, bool bMute)
{
	MIXERCONTROLDETAILS_BOOLEAN mcdbValues[2];
	MIXERCONTROLDETAILS mxcd;

	mcdbValues[0].fValue= static_cast<BOOL>(bMute);
	mxcd.cbStruct		= sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID	= m_vMutes[channel].dwControlID;
	mxcd.cChannels		= 1;
	mxcd.cMultipleItems	= 0;
	mxcd.cbDetails		= sizeof(MIXERCONTROLDETAILS_BOOLEAN);
	mxcd.paDetails		= mcdbValues;
	mixerSetControlDetails(reinterpret_cast<HMIXEROBJ>(m_hMixer), &mxcd, MIXER_SETCONTROLDETAILSF_VALUE);
}


bool CVolumeControl::GetMute(int channel)
{
	MIXERCONTROLDETAILS_BOOLEAN mcdbValues[2];
	MIXERCONTROLDETAILS mxcd;
	mxcd.cbStruct		= sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID	= m_vMutes[channel].dwControlID;
	mxcd.cChannels		= 1;
	mxcd.cMultipleItems	= 0;
	mxcd.cbDetails		= sizeof(MIXERCONTROLDETAILS_BOOLEAN);
	mxcd.paDetails		= mcdbValues;
	mixerGetControlDetails(reinterpret_cast<HMIXEROBJ>(m_hMixer), &mxcd, MIXER_GETCONTROLDETAILSF_VALUE);
	return (mcdbValues[0].fValue != 0);
}


void CVolumeControl::ChangeBalance(int channel, int value)
{
	int left, right, volume;
	if (value < -100) value = -100;
	if (value >  100) value =  100;
	GetVolume(channel, left, right);
	volume = (right > left) ? right : left;
	if (value > 0)
	{
		left = (volume / 100) * (100 - value);
		right = volume;
	}
	else
	{
		value *= -1;
		right = (volume / 100) * (100 - value);
		left = volume;
	}
	SetVolume(channel, left, right);
}

 


void CVolumeControl::ChangeVolume(int channel, int value)
{
	int balance;
	int left, right;

	if (value < 0) value = 0;
	if (value > 100) value = 100;

	GetVolume(channel, left, right);

	if (right > left)
	{
		if (left == 0)
			balance = 100;
		else
			balance = left / (right / 100);
		right = round(655.35 * value);

		if (right > 65535) right = 65535;
		if (right < 1) right = 0;

		left = (right / 100) * balance;
	}
	else if (right < left)
	{
		if (left == 0 || right == 0)
			balance = 100;
		else
			balance = right / (left / 100);
		left = round(655.35 * value);

		if (left > 65535) left = 65535;
		if (left < 1) left = 0;

		right = (left / 100) * balance;
	}
	else {
		left = round(655.35 * value);
		right = round(655.35 * value);
	}
	SetVolume(channel, left, right);
}


void CVolumeControl::VolumeUp(int channel, int nVolumeStep)
{
	int vol, left, right, balance;

	GetVolume(channel, left, right);
	if (right > left)
	{
		vol = round(right / 655.35 );
	}
	else
	{
		vol = round(left / 655.35 );
	}
	ChangeVolume(channel,vol+nVolumeStep);
}


void CVolumeControl::VolumeDown(int channel, int nVolumeStep)
{
	int vol, left, right, balance;

	GetVolume(channel, left, right);
	if (right > left)
	{
		vol = round(right / 655.35 );
	}
	else
	{
		vol = round(left / 655.35 );
	}
	ChangeVolume(channel,vol-nVolumeStep);
}


void CVolumeControl::BalanceLeft(int channel, int nBalanceStep)
{
	int balance, left, right;

	GetVolume(channel, left, right);
	if (left > right && left > 0)
	{
		balance = 100 * right / left;
		balance -= nBalanceStep;
		if (balance < 0) balance = 0;
		right = balance * (left / 100);
	}
	if (left <= right && right > 0)
	{
		balance = 100 * left / right;
		if (balance + nBalanceStep <= 100)
		{
			balance += nBalanceStep;
			left = balance * (right / 100);
		}
		else
		{
			balance += nBalanceStep;
			left = right;
			right = (100-(balance-100)) * (left / 100);
		}
	}
	SetVolume(channel, left, right);
}


void CVolumeControl::BalanceRight(int channel, int nBalanceStep)
{
	int balance, left, right;

	GetVolume(channel, left, right);
	if (right > left && right > 0)
	{
		balance = 100 * left / right;
		balance -= nBalanceStep;
		if (balance < 0) balance = 0;
		left = balance * (right / 100);
	}
	if (right <= left && left > 0)
	{
		balance = 100 * right / left;
		if (balance + nBalanceStep <= 100)
		{
			balance += nBalanceStep;
			right = balance * (left / 100);
		}
		else
		{
			balance += nBalanceStep;
			right = left;
			left = (100-(balance-100)) * (right / 100);
		}
	}
	SetVolume(channel, left, right);
}

///////////////////////////////////////////////////////////////////////

CVolumeControls::CVolumeControls(HWND hWnd)
{
	m_nCode = 0;
	m_Cnt = mixerGetNumDevs();
	m_Mixers = new CVolumeControl[m_Cnt];
	for (int i = 0; i < m_Cnt; i++)
	{
		if (m_nCode == 0)
		{
			m_Mixers[i].Init(hWnd, i);
			m_nCode = m_Mixers[i].m_nCode; 
		}
	}
	ReadRCSteps();
}

void CVolumeControls::ReInit()
{
	ReadRCSteps();
	for (int i = 0; i < m_Cnt; i++)
	{
		m_Mixers[i].ReInit();
	}
}

void CVolumeControls::ReadRCSteps()
{
	g_nVolumeStep	= GetRCInt("umSliderVolumeStep", 1);
	g_nBalanceStep	= GetRCInt("umSliderBalanceStep", 1);
}

CVolumeControls::~CVolumeControls()
{
	delete [] m_Mixers;
}

void CVolumeControls::GetVolume(int mixid, int channel, int &left, int &right)
{
	m_Mixers[mixid].GetVolume(channel, left, right);
}

void CVolumeControls::SetVolume(int mixid, int channel, int left, int right)
{
	if (CheckMixerRange(mixid))
		m_Mixers[mixid].SetVolume(channel, left, right);
}

void CVolumeControls::ChangeBalance(int mixid, int channel, int value)
{
	if (CheckMixerRange(mixid))
		m_Mixers[mixid].ChangeBalance(channel, value);
}

void CVolumeControls::ChangeVolume(int mixid, int channel, int value)
{
	if (CheckMixerRange(mixid))
		m_Mixers[mixid].ChangeVolume(channel, value);
}

void CVolumeControls::VolumeUp(int mixid, int channel, int nVolumeStep)
{
	if (CheckMixerRange(mixid))
		m_Mixers[mixid].VolumeUp(channel, nVolumeStep);
}

void CVolumeControls::VolumeDown(int mixid, int channel, int nVolumeStep)
{
	if (CheckMixerRange(mixid))
		m_Mixers[mixid].VolumeDown(channel, nVolumeStep);
}

void CVolumeControls::SetMute(int mixid, int channel, bool bMute)
{
	if (CheckMixerRange(mixid))
		m_Mixers[mixid].SetMute(channel, bMute);
}

bool CVolumeControls::GetMute(int mixid, int channel)
{
	if (CheckMixerRange(mixid))
		return (m_Mixers[mixid].GetMute(channel));
	else
		return false;
}

void CVolumeControls::BalanceLeft(int mixid, int channel, int nBalanceStep)
{
	if (CheckMixerRange(mixid))
		m_Mixers[mixid].BalanceLeft(channel, nBalanceStep);
}

void CVolumeControls::BalanceRight(int mixid, int channel, int nBalanceStep)
{
	if (CheckMixerRange(mixid))
		m_Mixers[mixid].BalanceRight(channel, nBalanceStep);
}

bool CVolumeControls::CheckMixerRange(int mixid)
{
	if ((mixid < 0) || (mixid > m_Cnt))
	{
		utils::ErrorBox("Mixer ID is out of range");
		return false;
	}
	return true;
}
